function [ is_successful ] = upload( p_config, p_nemo_data )
%ftp_server_upload Uploads profiles to the ftp.awi.de server as user argo.
%   If p_nemo_data is given, only the given profiles will be uploaded.
%   Otherwise all profiles on the harddisk will be uploaded to the server
%   as defined in p_config.
%   Some default names are being ignored.
%   Existing files on the server will be overwritten. Files
%   existing ONLY on the server will not be touched.
%
%   Parameters:
%   p_config:           The nemo configuration.
%   p_nemo_data:        Optional, the nemo data that will be uploaded to
%                       the server.


%% Initialize  return variables
is_successful = false;

%% Check input parameter
if (~isstruct(p_config) || ~isfield(p_config, 'FTP')) % display a warning
    disp([ mfilename ': Please provide valid nemo configuration!']);
    return;
end

if (exist('p_files_to_upload', 'var') && ~iscell(p_nemo_data))
    disp([ mfilename ': Please provide a cell of strings in p_files_to_upload!']);
    return;
end

%% Set up server connection

try % connecting to ftp server
    ftp_server = ftp( ...
        p_config.FTP.server, ...
        p_config.FTP.username, ...
        p_config.FTP.password ...
    );
catch error
    disp([mfilename sprintf(': Failed to connect to server!\n') error]);
    return;
end

% change to correct folder if necessary
if (~strcmp(cd(ftp_server), p_config.FTP.remote_folder))
    cd(ftp_server, p_config.FTP.remote_folder);
end

%% Start upload process

if exist('p_nemo_data', 'var') % upload files that have been given
    for i = 1:length(p_nemo_data)
        for k = 1:length(p_nemo_data{i})
            profile_filename = [ ...
                sprintf('%04d', p_nemo_data{i}{k}.FLOAT_IDENTIFICATION.float_serial_number) ...
                '_' ...
                sprintf('%04d', p_nemo_data{i}{k}.PROFILE_TECHNICAL_DATA.xmit_profile_number) ...
                '.profile'
                ];
            mkdir( ...
                ftp_server, ...
                [ ...
                    p_config.FTP.remote_folder ...
                    p_config.FTP.folder_separator ...
                    sprintf('%04d', p_nemo_data{i}{k}.FLOAT_IDENTIFICATION.float_serial_number) ...
                ] ...
            );
            cd( ...
                ftp_server, ...
                [ ...
                    p_config.FTP.remote_folder ...
                    p_config.FTP.folder_separator ...
                    sprintf('%04d', p_nemo_data{i}{k}.FLOAT_IDENTIFICATION.float_serial_number) ...
                ] ...
            );
            mput(ftp_server, ...
                 [ ...
                    p_config.OUTPUT_FILES.output_path ...
                    sprintf('%04d', p_nemo_data{i}{k}.FLOAT_IDENTIFICATION.float_serial_number) ...
                    p_config.GENERAL.folder_separator ...
                    profile_filename ...
                ] ...
            );
        end
    end
    
else

    directory_content = dir(p_config.OUTPUT_FILES.output_path); % should contain all float directories
    
    % run through every content of the folder
    for current = 1:size(directory_content, 1)
        if (directory_content(current).isdir)
            % create directory
            mkdir(ftp_server, ...
                [ ...
                p_config.FTP.remote_folder ...
                p_config.FTP.folder_separator ...
                directory_content(current).name ...
                ] ...
                );
            % switch to it on the server side
            cd(ftp_server, [p_config.FTP.remote_folder p_config.FTP.folder_separator directory_content(current).name]);
            % upload all .profile files
            current_profiles = dir([p_config.OUTPUT_FILES.output_path directory_content(current).name p_config.GENERAL.folder_separator '*.profile']);
            for i = 1:size(current_profiles, 1)
                if (strcmp(current_profiles(i).name, 'Startup_Test_Message.profile') ...
                        || strcmp(current_profiles(i).name, 'MTM_Report_Message.profile') ...
                        || strcmp(current_profiles(i).name, 'nemo_0NaN_NaN.profile'))
                    continue % skip some default names
                end
                % upload the file
                mput(ftp_server, [p_config.OUTPUT_FILES.output_path ...
                    directory_content(current).name p_config.GENERAL.folder_separator current_profiles(i).name]);
            end
        end
    end

end




%% Close server connection

try % closing the connection
    close(ftp_server);
catch error
    disp([mfilename sprintf(': Closing connection to server failed:\n') getReport(error)]);
end

is_successful = true;

return


end

