function [ profile ] = load( p_filename )
%LOAD Loads a .profile file into memory.
%   
%   Returns: A struct containing all parameter and data that has been read from
%   the datafile. WARNING: Does not check if something went wrong while reading
%   the file.
%   
%   Parameters:
%   p_filename:    Absolute filename of the file to read.
%

%% TODO:
% - remove sections and make this function recognize automatically which type
%   of variable should be used, same as in nemo.profile.save().

%% Initialize input parameter
profile = false;

%% Parameter check
parameter_error = false;

if (~ischar(p_filename))
    warning([mfilename ': Please check input parameter. p_filename is not a string!']);
    parameter_error = true;
else
    % get filename details
    [pathstr, name, ext] = fileparts(p_filename);
    if (~strcmp(ext, '.profile'))
        warning([mfilename ': Please check input parameter. Wrong file extension! Should be .profile!']);
        parameter_error = true;
    end
end

if parameter_error
    return; % return if parameter check has not been passed successfully
else
    clear parameter_error;
end

%% Load profile into variable

% initialize all required variables
profile = {};
current_section = '';
comment_char = '%'; % character to indicate comments
section_char = '['; % character indicating new section

% open file
file_id = fopen(p_filename);
fseek(file_id, 0, 'bof');


% read file line by line
current_line = fgetl(file_id);
while (ischar(current_line))
    if length(current_line) == 0
        current_line = fgetl(file_id);
        continue
    end
    
    switch current_line(1)
        case section_char % this indicates a new section
            current_line = strtrim(current_line);
            current_section = current_line(2:end-1);
            read_section
        otherwise
            current_line = fgetl(file_id);
    end

end

fclose(file_id);

    function read_section
        % read complete section
        current_line = fgetl(file_id);
        
        if isempty(current_line)
            % initialize an empty array to prevent errors when trying to
            % access in another function
            profile.(current_section) = [];
        end
        
        while ischar(current_line) && ~isempty(current_line) ...
                && ~strcmp(current_line(1), section_char)
            
            if strcmp(current_line(1), comment_char) % ignore commented lines
                current_line = fgetl(file_id);
                continue
            end
		
			% if it contains a comment char anywhere, just take the string
            % before it
            found_comment = strfind(current_line, comment_char);
            if ~isempty(found_comment)
                current_line = current_line(1:found_comment(1)-1);
            end
            
            switch (current_section)
                case { ...
                        'FLOAT_IDENTIFICATION', ...
                        'OVERALL_MISSION_INFORMATION', ...
                        'DEPLOYMENT_INFO', ...
                        'PROFILE_TECHNICAL_DATA', ...
                        'BOTTOM_VALUES_DURING_DRIFT', ...
                        'PROFILE_HEADER', ...
                        'QUALITY_CONTROL_HEADER', ...
                        'STARTUP_MESSAGE', ...
                        'SECOND_ORDER_INFORMATION' ...
                        }
                    % get variable name and value
                    [token, remain] = strtok(current_line);
                    % try to convert remain to a number
                    remain_as_num = str2num(strtrim(remain));
                    if (~isempty(remain_as_num))
                        profile.(current_section).(token(2:end)) = remain_as_num;
                    else
                        profile.(current_section).(token(2:end)) = strtrim(remain);
                    end
                case { ...
                        'RAFOS_VALUES_FORMAT', ...
                        'PROFILE_DATA_FORMAT', ...
                        'SURFACE_GPS_DATA_FORMAT', ...
                        'IRIDIUM_POSITIONS_FORMAT', ...
                        'IRIDIUM_DATA_FORMAT', ...
                        'PROFILE_DATA_HEADER' ...
                        }
                    % remove -format, split afterwards
                    [token, remain] = strtok(current_line);
                    if (~strcmp(token, '-format'))
                        profile.(current_section) = [ token, ...
                            strsplit(strtrim(remain), {' ', '\t'}, 'CollapseDelimiters', true)];
                    else
                        profile.(current_section) = strsplit( ...
                            strtrim(remain), {' ', '\t'}, 'CollapseDelimiters', true);
                    end
                    %break
                case { ...
                        'RAFOS_VALUES', ...
                        'PROFILE_DATA', ...
                        'SURFACE_GPS_DATA', ...
                        'IRIDIUM_POSITIONS', ...
                        'IRIDIUM_DATA' ...
                        }
                    % store into matrix
                    % read values and append them into the same var as long
                    % as no empty line occured (convention)
                    profile.(current_section) = [];
                    while (ischar(current_line) && ~strcmp(current_line, ''))
                        profile.(current_section) = [ ...
                            profile.(current_section); ...
                            str2double(strsplit(current_line, {'\t', ' '}, 'CollapseDelimiters', true)) ...
                            ];
                        current_line = fgetl(file_id);
                    end
                    break
                    
            end
            
            current_line = fgetl(file_id);
        end
    end


end

